;----------------------------------------------------------;
; Online Mode

chk_online:
	 rcall	chk_rcvr
	breq	PC+4
	 rcall	receive
	cpi	AL, 13
	breq	PC+2
	ret

online:
	ldiw	Z, ml_online*2		;"ONLINE"
	 rcall	lcd_fprin_clr		;/
	ldiw	Z, m_start*2		;Start up message
	 rcall	dp_str			;/

mon_loop:
	 rcall	crlf		; Display command prompt
	ldi	AL, '>'		; 
	 rcall	xmit		; /
	 rcall	get_line	; Input command
	ldiw	X, LineBuf	; X -> command line
	ld	CL, X+		; BH = command char
	 rcall	caps		; /
	cpi	CL, 0x0d	; Null line?
	breq	mon_loop	;  yes, returen
	cpi	CL, 'P'		; Play?
	breq	do_play		;  yes, do
	cpi	CL, 'D'		; Dump memory?
	rjeq	do_dump		;  yes, do
	cpi	CL, 'V'		; Vector out?
	rjeq	do_vect		;  yes, do
	cpi	CL, 'F'		; File operation?
	rjeq	do_file		;  yes, do

cmd_err: rcall	crlf		;Display error
	 rcall	space		;
	sbiw	XL, 1		;
	cpi	XL, low(LineBuf);
	brne	PC-3		;
	ldiw	Z, m_error*2	;
	rjmp	cmd_mes
cmd_nofile:
	ldiw	Z, m_nofile*2
	rjmp	cmd_mes
cmd_fserr:
	ldiw	Z, m_fserr*2
cmd_mes: rcall	dp_str
	rjmp	mon_loop


m_error:	.db	"^error", 0
m_nofile:	.db	13,10, "File not found.", 0
m_fserr:	.db	13,10, "I/O Error.", 0



;--------------------------------------;
; Play a vector file

do_play:
	adiw	XL, 1		;Open given file
	ld	AL, X		;
	cpi	AL, ' '		;
	brcs	cmd_err		;
	breq	PC-5		;
	movw	ZL, XL		;
	ldiw	Y, File1	;
	ldiw	A, Buff1	;
	 rcall	file_open	;
	brne	cmd_fserr	;
	cpw	Y, _0		;
	breq	cmd_nofile	;/
	 rcall	play
	rjmp	mon_loop



;--------------------------------------;
; Dump Sector

do_dump:
	ld	CL, X
	 rcall	caps
	cpi	CL, 'I'
	breq	dd_info

dd_sect:
	 rcall	get_val
	rjcs	cmd_err
	ldiw	Y, DumpAdr
	brne	PC+7
	lddw	A, Y+0
	ldd	BL, Y+2
	subiw	A, -1
	sbci	BL, -1
	stdw	Y+0, A
	std	Y+2, BL

	ldiw	Z, md_sect*2
	 rcall	dp_str
	lddw	A, Y+0
	ldd	BL, Y+2
	 rcall	dp_dec24
	ldiw	X, Buff0
	lddw	A, Y+0
	ldd	BL, Y+2
	 rcall	file_get_sect0
	rjcs	cmd_fserr

	ldiw	Y, Buff0	;Dump sector buffer
	clrw	X		;
	ldi	CL, 16		;
	 rcall	put_dump	;
	cpi	XH, high(512)	; End of block ?
	brcs	PC-3		;  no, continue

	rjmp	mon_loop

dd_info:
	ldiw	Y, LineBuf	;Read CSD, CID, OCR
	movw	XL, YL		;
	 rcall	mmc_read_info	;
	rjne	cmd_fserr	;/
	clrw	X		;Dump CSD, CID, OCR
	ldi	CL, 16		;
	 rcall	put_dump	;
	ldi	CL, 16		;
	 rcall	put_dump	;
	ldi	CL, 4		;
	 rcall	put_dump	;/
	ldiw	Y, FatBase	;Dump FAT16 information
	ldi	CL, 12		;
	 rcall	put_dump	;/

	rjmp	mon_loop


put_dump:
	push	CL
	 rcall	crlf		; ---- Line loop
	movw	AL, XL		; Line header (address)
	 rcall	dp_word		; 
	 rcall	space		; /
	 rcall	space		;---- Byte loop
	ld	AL, Y+		; Data
	 rcall	dp_byte		; /
	adiw	XL, 1		;
	dec	CL		; End of line ?
	brne	PC-5		;  no, continue
	 rcall	space		;
	 rcall	space		;
	pop	CL
	sub	YL, CL
	sbc	YH, _0
	ld	AL, Y+		; Data
	cpi	AL, ' '		;
	brcs	PC+3		;
	cpi	AL, 0x80	;
	brcs	PC+2		;
	ldi	AL, '.'		;
	 rcall	xmit		; /
	dec	CL		; End of line ?
	brne	PC-8		;  no, continue
	ret


md_sect:.db	13,10, "Sector: ", 0



;--------------------------------------------------;
; Vector output

do_vect:
	 rcall	get_val		;T4 = X position
	breq	dv_off		;
	movw	T4L, AL		;/
	 rcall	get_val		;T6 = Y position
	breq	dv_off		;
	movw	T6L, AL		;/
	 rcall	get_val		;T = mod
	brne	PC+2		;
	clr	AL		;
	bst	AL, 0		;/
	movw	AL, T4L		;A = (T4 + 2048) & 4095
	addi	AH, high(2048)	;
	andi	AH, high(4096-1);/
	movw	BL, T6L		;B = (T6 + 2048) & 4095
	addi	BH, high(2048)	;
	andi	BH, high(4096-1);/
	bld	BH, 7
	 rcall	set_vector

	in	AL, PORTE
	bst	CL, 0
	bld	AL, LDMOD
	out	PORTE, AL

	rjmp	mon_loop
dv_off:
	sbi	PORTE, LDMOD
	rjmp	mon_loop




;--------------------------------------------------;
; File functions

do_file:
	ld	CL, X
	 rcall	caps
	cpi	CL, 'L'
	rjeq	df_list
	cpi	CL, 'T'
	breq	df_type
	cpi	CL, 'B'
	breq	df_type2
	rjmp	cmd_err


df_type:
	adiw	XL, 1
	ld	AL, X
	cpi	AL, ' '
	rjcs	cmd_err
	breq	PC-5

	movw	ZL, XL
	ldiw	Y, File1
	ldiw	A, Buff1
	 rcall	file_open
	rjne	cmd_fserr
	cpw	Y, _0
	rjeq	cmd_nofile

;	ldiw	A, 1000
;	clrw	B
;	 rcall	file_seek
;	rjne	cmd_fserr

	 rcall	crlf
dft_l1:
	ldiw	Z, LineBuf+2
	ldiw	C, 1
	 rcall	file_read
	rjne	cmd_fserr
	cpw	C, _0
	rjeq	mon_loop
	ld	AL, Z
	 rcall	xmit
	rjmp	dft_l1


df_type2:
	adiw	XL, 1
	ld	AL, X
	cpi	AL, ' '
	rjcs	cmd_err
	breq	PC-5

	movw	ZL, XL
	ldiw	Y, File1
	ldiw	A, Buff1
	 rcall	file_open
	rjne	cmd_fserr
	cpw	Y, _0
	rjeq	cmd_nofile
	 rcall	crlf
dft_l2:
	ldiw	Z, Buff1
	ldiw	C, 512
	 rcall	file_read_block
	rjne	cmd_fserr
	cpw	C, _0
	rjeq	mon_loop
dft_l3:
	cpw	C, _0
	breq	dft_l2
	ld	AL, Z+
	 rcall	xmit
	subiw	C, 1
	rjmp	dft_l3


df_list:
	clrw	T2		;Count = 0
	clrw	T4		;Total Size = 0
	clrw	T6		;/
	ldiw	D, 0		;Reset file search
	 rcall	file_dir_init	;/
dfl_l1:
	 rcall	file_dir_next	;Get next file
	brne	dfl_err		;/
	cpw	Z, _0		;No file?
	breq	dfl_end		;/
	ldd	AL, Z+11	;Is V/S/H?
	andi	AL, 0b00001110	;
	brne	dfl_l1		;/
	 rcall	crlf		;CRLF
	movw	XL, ZL		;Name
	ldi	CL, 11		;
	ld	AL, X+		;
	 rcall	xmit		;
	dec	CL		;
	brne	PC-3		;/
	ldi	CL, 4		;"    "
	ldi	AL, ' '		;
	 rcall	xmit		;
	dec	CL		;
	brne	PC-3		;/
	ldd	AL, Z+11	;Is a file/directory?
	sbrc	AL, 4		;
	rjmp	dfl_dir		;
	lddw	A, Z+28		; Size
	lddw	B, Z+30		;
	addw	T2, A		;
	adcw	T4, B		;
	 rcall	dp_dec32	; /
	rjmp	dfl_nxt
dfl_dir:ldiw	Z, dfl_str2*2	;<DIR>
	 rcall	dp_str		;/
dfl_nxt:sec			;Count++
	adcw	T6, _0		;/
	rjmp	dfl_l1
dfl_end:
	ldiw	Z, dfl_str1*2	;CRLF
	 rcall	dp_str		;
	movw	AL, T6L		;Files
	 rcall	dp_dec16	;
	 rcall	dp_str		;/
	movw	AL, T2L		;Bytes
	movw	BL, T4L		;
	 rcall	dp_dec32	;
	 rcall	dp_str		;/
dfl_err:
	rjmp	mon_loop

dfl_str1:	.db	13,10, "  ",0, " files,  ",0," bytes",0
dfl_str2:	.db	"<DIR>",0


;---------------------------------------------------;
; Get a Line from Host
;
; Call: none
; Ret:  LineBuf

get_line:
	ldiw	X, LineBuf
	ldi	BH, 0
rl_lp:	 rcall	receive
	breq	rl_lp
	st	X, AL
	cpi	AL, 0x0d	; CR
	brne	PC+2
	rjmp	xmit
	cpi	AL, 0x08	; BS
	brne	PC+7
	cpi	BH, 0
	breq	rl_lp
	 rcall	xmit
	sbiw	XL, 1
	dec	BH
	rjmp	rl_lp
	cpi	AL, ' '		; SP
	brcs	rl_lp
	cpi	BH, 40-1
	brcc	rl_lp
	 rcall	xmit
	adiw	XL, 1
	inc	BH
	rjmp	rl_lp


caps:	cpi	CL, 'a'		;CAPS
	brcs	PC+4		;
	cpi	CL, 'z'+1	;
	brcc	PC+2		;
	subi	CL, 0x20	;/
	ret



;--------------------------------------------------------;
; Send ROM string
;
; Call: Z = top of the string (ASCIZ)
; Ret:  Z = next string
; Broken: AL

dp_str:	lpm	AL, Z+
	tst	AL
	brne	PC+2
	ret
	 rcall	xmit
	rjmp	dp_str



;--------------------------------------------------------;
; Send a byte value in two chars of hexdecimal string
;
; Call: AL = 8 bit value to be sent (hex)
;       A = 16 bit value to be sent (hex)
;       BL:A = 24 bit value to be sent (decimal)
; Broken: CL, B, A, T0L (decimal)

dp_word:
	push	AL
	mov	AL, AH
	 rcall	dp_byte
	pop	AL
dp_byte:
	swap	AL
	 rcall	PC+2
	swap	AL
	push	AL
	andi	AL,0x0f
	cpi	AL, 10
	brcs	PC+2
	addi	AL, 7
	addi	AL, '0'
	 rcall	xmit
	pop	AL
	ret

dp_bits:
	ldi	CL, 8
dp_bits2:
	clr	T0L
	ldi	AL, '0'
	lsr	AH
	adc	AL, _0
	push	AL
	inc	T0L
	dec	CL
	brne	PC-6
	pop	AL
	 rcall	xmit
	dec	T0L
	brne	PC-3
	ret

dp_dec16:
	clr	BL
dp_dec24:
	clr	BH
dp_dec32:
	clr	T0L
	inc	T0L
	clr	CL
	ldi	CH, 32
	lslw	A
	rolw	B
	rol	CL
	cpi	CL, 10
	brcs	PC+3
	subi	CL, 10
	inc	AL
	dec	CH
	brne	PC-10
	addi	CL, '0'
	push	CL
	cpw	A, _0
	cpcw	B, _0
	brne	PC-20
	pop	AL
	 rcall	xmit
	dec	T0L
	brne	PC-3
	ret



;---------------------------------------------------------;
; Get hexdecimal value
;
; Call: X = string pointer  "12345 $3FF0 -100"
; Ret:  X = updated
;         if    C=1: Error
;         elsif Z=1: End of line, BL:A = 0
;         else:      BL:A = esult
; Broken: BH, C, T2, T0


get_val:
	ldiw	A, 0
	ldi	BL, 0
	ld	CL, X+
	cpi	CL, ' '
	brcs	gv_n
	breq	PC-3
	ldi	CH, 10
	clt
	cpi	CL, '-'
	brne	PC+3
	set
	rjmp	PC+4
	cpi	CL, '$'
	brne	PC+3
	ldi	CH, 16
gv_l:	ld	CL, X+
	 rcall	caps
	cpi	CL, ' '+1
	brcs	gv_e
	subi	CL, '0'
	brcs	gv_x
	cpi	CL, 10
	brcs	PC+4
	subi	CL, 7
	cpi	CL, 10
	brcs	gv_x
	cp	CL, CH
	brcc	gv_x
	mul	AL, CH
	movw	T2L, T0L
	mul	BL, CH
	mov	BL, T0L
	mul	AH, CH
	movw	AL, T2L
	add	AH, T0L
	adc	BL, T0H
	add	AL, CL
	adc	AH, _0
	adc	BL, _0
	rjmp	gv_l
gv_x:	sec
	rjmp	PC+2
gv_n:	clc
	sez
	ret
gv_e:	brtc	PC+7
	comw	A
	com	BL
	subiw	A, 1
	sbci	BL, 0
	sbiw	XL, 1
	ret



;---------------------------------------------------------------;
; Strings

ml_online:	.db	"ONLINE", 0
m_start:	.db	13,10
		.db	"Laser Display Controller R0.2 ", 13,10, 0


