;---------------------------------------------------------------;
; Space, CRLF

crlf:
	ldi	AL, 13
	 rcall	xmit
	ldi	AL, 10
	rjmp	xmit

space:
	ldi	AL,' '


;---------------------------------------------------------------;
; UART control driver

xmit:	; Transmit AL
	sbis	UCSR0A, UDRE0
	rjmp	PC-1
	out	UDR0, AL
	ret

chk_rcvr:
	ldsw	A, RxBuff
	cp	AL, AH
	ret



receive:; Receive a char into AL. (ZR=no data)
	ldiw	Y, RxBuff
	cli
	ldd	AH, Y+0
	ldd	AL, Y+1
	cp	AH, AL
	breq	PC+9
	mov	AL, AH
	inc	AL
	andi	AL, 31
	std	Y+0, AL
	add	YL, AH
	adc	YH, _0
	ldd	AL, Y+2
	clz
	sei
	ret


isr_usart0_rx:
	push	AL
	in	AL, SREG
	push	BL
	pushw	A
	pushw	Y
	in	BL, UDR0
	ldiw	Y, RxBuff
	ldd	AL, Y+0
	ldd	AH, Y+1
	inc	AH
	andi	AH, 31
	cp	AH, AL
	breq	PC+7
	std	Y+1, AH
	dec	AH
	andi	AH, 31
	add	YL, AH
	adc	YH, _0
	std	Y+2, BL
	popw	Y
	popw	A
	pop	BL
	out	SREG, AL
	pop	AL
	reti



;----------------------------------------------------------;
; Key driver


get_key:
	lds	AL, KeyCmd
	andi	AL, K_MASK
	breq	get_key
get_key_nw:
	lds	AL, KeyCmd
	andi	AL, K_MASK
	breq	PC+8
	sts	KeyCmd, _0
	lds	AH, KeyCmd+1
	andi	AH, K_MASK
	swap	AH
	or	AL, AH
	ret


isr_tc0_com:
	push	T0L
	in	T0L, SREG
	sei
	pushw	A

	in	AL, PINF
	com	AL
	andi	AL, K_MASK
	lds	AH, KeyCmd+2
	sts	KeyCmd+2, AL
	cp	AL, AH
	brne	isc_0e
	lds	AH, KeyCmd+1
	sts	KeyCmd+1, AL
	eor	AH, AL
	and	AH, AL
	breq	isc_0e
	sts	KeyCmd+0, AH

isc_0e:
	popw	A
	out	SREG, T0L
	pop	T0L
	reti



;-------------------------------------------------------;
; Die with dieing message
;
;Call: Z = Pointer to the message
;      T14L = Error code (0..9)

die:
	 rcall	lcd_fprin_clr
	mov	AL, T14L
	 rcall	lcd_prin_val
	rjmp	PC


die_mmcerr:	.db	"MMC Err ", 0
die_fserr:	.db	"FS Err ", 0
die_ioerr:	.db	"IO Err ", 0



;----------------------------------------------------------;
; LCD control
;
; Data:PD[3..0], RS:PD4, E:PD5
;

.equ	LCDRS	= PD4
.equ	LCDEN	= PD5


lcd_init:
	 rcall	delay_5ms
	 rcall	delay_5ms
	 rcall	delay_5ms
	cbi	PORTD, LCDRS	; RS=0
	ldi	AL, 0b00110000
	 rcall	lcd_wr_n
	 rcall	delay_5ms
	ldi	AL, 0b00110000
	 rcall	lcd_wr_n
	 rcall	delay_5ms
	ldi	AL, 0b00110000
	 rcall	lcd_wr_n
	 rcall	delay_5ms
	ldi	AL, 0b00100000
	 rcall	lcd_wr_n
	ldi	AL, 0b00101000	;DL=0, N=1, F=0
	 rcall	lcd_wr_cmd
	ldi	AL, 0b00001100	;D=1, C=0, B=0
	 rcall	lcd_wr_cmd
	ldi	AL, 0b00000110	;I/D=1, S=0
	 rcall	lcd_wr_cmd
	rjmp	lcd_clear



lcd_prin_val:
	clr	CH
	inc	CH
	ldi	CL, 8
	clr	AH
	lslw	A
	cpi	AH, 10
	brcs	PC+3
	subi	AH, 10
	inc	AL
	dec	CL
	brne	PC-7
	addi	AH, '0'
	push	AH
	cpi	AL, 0
	brne	PC-14
	pop	AL
	 rcall	lcd_wr_chr
	dec	CH
	brne	PC-3
	ret



lcd_mprin_clr:
	 rcall	lcd_clear
lcd_mprin:
	ld	AL, Z+
	tst	AL
	brne	PC+2
	ret
	 rcall	lcd_wr_chr
	rjmp	lcd_mprin


lcd_fprin_clr:
	 rcall	lcd_clear
lcd_fprin:
	lpm	AL, Z+			;Get a char
	tst	AL			;Zero?
	brne	PC+2			;/
	ret
	 rcall	lcd_wr_chr		;
	rjmp	lcd_fprin		;/


lcd_clear:
	stsi	Lcdp, 8
	ldi	AL, 0x01
	 rcall	lcd_wr_cmd
	rjmp	delay_5ms


lcd_wr_chr:
	sbi	PORTD, LCDRS
	 rcall	lcd_delay
	 rcall	lcd_wr_n
	 rcall	lcd_wr_n
	lds	AL, Lcdp
	dec	AL
	sts	Lcdp, AL
	breq	PC+2
	ret
	ldi	AL, 0xC0
lcd_wr_cmd:
	cbi	PORTD, LCDRS
	 rcall	lcd_delay
	 rcall	lcd_wr_n
lcd_wr_n:
	swap	AL
	push	AL
	andi	AL, 0b00001111
	in	AH, PORTD
	andi	AH, 0b11110000
	or	AH, AL
	out	PORTD, AH
	pop	AL
	sbi	PORTD, LCDEN
	rjmp	PC+1
	cbi	PORTD, LCDEN
	ret

lcd_delay:
	ldi	AH, 848/4	;Delay 53us
	dec	AH		;
	nop			;
	brne	PC-2		;/
	ret

delay_5ms:			;Delay 5ms
	ldiw	X, 80000/4
	sbiw	XL, 1
	brne	PC-1
	ret


