;----------------------------------------------------------------------------;
; Laser module temperature controller
;----------------------------------------------------------------------------;
;

.include "tn26def.inc"	; This file is contained in "AVR Family Assembler".
.include "avr.inc"

.equ	BPS	= 33	; 9.6kbps @1.00MHz (UART.ASMQ)
.equ	MISO	= 1	; Change these value when use at 8/28 pin device.
.equ	MOSI	= 0	; /

.equ	CT_PP	= 10
.equ	CT_PN	= 5
.equ	CT_MAX	= 180
.equ	CT_MIN	= -120


.def	_0	= r15
.def	_Tmr	= r14
.def	_SetH	= r13
.def	_SetL	= r12
.def	_ItH	= r11
.def	_ItL	= r10
.def	_TmpH	= r9
.def	_TmpL	= r8


.def	_Flags	= r25	; b0: 1sec
			; b1: Failed
			; b2: Saturated



;----------------------------------------------------------;
; SRAM̈`

.dseg
	.org	RAMTOP

LineBuf:.byte	16	; R}h̓obt@



;----------------------------------------------------------;
; vOR[h

.cseg
	rjmp	reset
	rjmp	isr_int0
	rjmp	isr_iopin
	rjmp	isr_tc1cma
	rjmp	isr_tc1cmb
	rjmp	isr_tc1ovf
	rjmp	isr_tc0ovf
;	rjmp	isr_usist
;	rjmp	isr_usiovf
;	rjmp	isr_eeprom
;	rjmp	isr_comp
;	rjmp	isr_adc

isr_iopin:
isr_tc1cma:
isr_tc1cmb:
isr_tc1ovf:

;----------------------------------------------------------;
; 

reset:
	clr	_0
	outi	SP,RAMEND		; SP 

	outi	PORTA, 0b10111101	;PortA 
	outi	DDRA,  0b01000011	;/
	outi	PORTB, 0b11100101	;PortB 
	outi	DDRB,  0b00011000	;/

	sbi	DDRB, MISO		; MISOo

	outi	TCCR0, 0b00000101	;TC0.ck = 976Hz
	outi	TIMSK, 0b00000010

	outi	TCCR1A, 0b00100001	;OC1B = PWM
	outi	OCR1C, 255		;/
	outi	TCCR1B, 0b00000001	;TC1.ck = 1MHz

	outi	MCUCR, 0b00000001	;ISC0[1:0] = 01
	outi	GIMSK, 0b01000000	;Enable INT0

	clr	_Flags

	sbis	PINB, MOSI
	rjmp	monitor

run:
	 rcall	load_parm
	clrw	_It
	sei
	sbrc	_Flags, 1
	rjmp	failed

;----------------------------------------------------------;
; x䃁C

main:
	sbrs	_Flags, 0	;1sec
	rjmp	main		;
	cbr	_Flags, bit0	;/

	 rcall	get_temp	;A = Temperature
	sbrc	_Flags, 1	;
	rjmp	failed		;
	movew	_Tmp, A		;/
	 rcall	put_temp	;Display temperature
	ldi	AL, '/'		;
	 rcall	xmit		;/

	subw	_Tmp, _Set	;A = error
	movew	A, _Tmp		;/
	 rcall	check_temp	;LD output enable/inhibit
	movew	B, _It		;I term
	ldi	CL, 5		;
	asrw	B		;
	dec	CL		;
	brne	PC-3		;
	addw	A, B		;/

	 rcall	set_cooler	;Drive cooler

	 rcall	dp_dec		;Display output
	ldiw	Z, m_crlf*2	;
	 rcall	dp_str		;/

	movew	A, _Tmp		;Integration
	sbrs	_Flags, 3
	rjmp	PC+8
	sbic	PORTB, 4
	rjmp	PC+4
	sbrs	AH, 7
	rjmp	PC+6
	rjmp	PC+3
	sbrc	AH, 7
	rjmp	PC+3
	addw	_It, A

	rjmp	main



failed:
	cbi	PORTA, 1
	cbi	PORTB, 4
	out	OCR1B, _0
	rjmp	PC


check_temp:
	ldiw	B, 15		;Check if |error| > 1.5 deg.c
	cpw	A, B		;
	brge	PC+6		;
	ldiw	B, -15		;
	cpw	A, B		;
	brge	PC+3		;
	sbr	_Flags, bit2	;
	cbi	PORTA, 1	;/

	ldiw	B, 10		;Check if |error| < 1 deg.c
	cpw	A, B		;
	brge	PC+8		;
	ldiw	B, -10		;
	cpw	A, B		;
	brlt	PC+3		;
	cbr	_Flags, bit2	;
	 rcall	isr_int0	;/

	ret


set_cooler:
	ldi	BH, CT_PP	;Contorl gain
	sbrc	AH, 7		;
	ldi	BH, CT_PN	;
	 rcall	mul1608s	;/
	cbr	_Flags, bit3	;Check if saturated
	ldiw	B, CT_MAX	;
	cpw	A, B		;
	brlt	PC+4		;
	sbr	_Flags, bit3	;
	movew	A, B		;
	ldiw	B, CT_MIN	;
	cpw	A, B		;
	brge	PC+4		;
	sbr	_Flags, bit3	;
	movew	A, B		;/
	sbrs	AH, 7		;Set cooler power
	rjmp	PC+3		;
	sbi	PORTB, 4	;
	rjmp	PC+2		;
	cbi	PORTB, 4	;
	out	OCR1B, AL	;/
	ret


mul1608s:
	sbrs	AH, 7
	rjmp	mul1608
	comw	A
	subiw	A, -1
	 rcall	mul1608
	comw	A
	subiw	A, -1
	ret

mul1608:			; A = BH * A;
	ldi	CL, 17
	sub	BL, BL
	brcc	PC+2
	add	BL, BH
	ror	BL
	rorw	A
	dec	CL
	brne	PC-6
	ret



;----------------------------------------------------------;
; R}hs[v

monitor:
	ldiw	Z,m_start*2	; Display start up message
	 rcall	dp_str		; /

mon_loop:
	ldiw	Z,m_crlf*2	; Display command prompt
	 rcall	dp_str		; 
	ldi	r16,'%'		; 
	 rcall	xmit		; /
	 rcall	get_line	; Input command
	ldiw	X,LineBuf	; X -> command line
	ld	r19,X+		; r19 = command char
	 rcall	caps		; /
	cpi	r19,0x0d	; Null line ?
	breq	mon_loop	;  yes, returen
	cpi	r19,'S'		; Set temp ?
	breq	do_settemp	;  yes, do
	cpi	r19,'C'		; Cooler output ?
	breq	do_out		;  yes, do
	cpi	r19,'T'		; Temperature ?
	rjeq	do_temp		;  yes, do
	cpi	r19,'G'		; Start ?
	rjeq	run		;  yes, do

cmd_err:ldiw	Z,m_error*2	; Syntax error
	 rcall	dp_str		; /
	rjmp	mon_loop



;--------------------------------------;
; xݒ

do_settemp:
	 rcall	get_valu
	breq	cmd_err

	out	EEDR, AL
	outi	EEAR, 0
	sbi	EECR, EEMWE
	sbi	EECR, EEWE
	sbic	EECR, EEWE
	rjmp	PC-1

	rjmp	mon_loop


load_parm:
	outi	EEAR, 0
	sbi	EECR, EERE
	in	AL, EEDR
	cpi	AL, 40
	brcs	PC+2
	sbr	_Flags, bit1
	clr	AH
	clr	BL
	ldi	BH, 10
	 rcall	mul1608
	movew	_Set, A
	ret


;--------------------------------------;
; Coolero

do_out:
	 rcall	get_valu
	breq	cmd_err
	out	OCR1B, AL
	rjmp	mon_loop


;--------------------------------------;
; Display temperature

do_temp:
	 rcall	get_temp
	sbrs	_Flags, 1
	 rcall	put_temp
	rjmp	mon_loop

put_temp:
	ldi	ZH, ' '		;Sign
	sbrs	AH, 7		;
	rjmp	PC+6		;
	ldi	ZH, '-'		;
	comw	A		;
	adc	AL, _0		;
	adc	AH, _0		;/
	 rcall	div10		;x1
	addi	BL, '0'		;
	mov	CL, BL		;
	ldi	ZL, 0		;
	 rcall	div10		;
	addi	BL, '0'		;
	inc	ZL		;
	st	Z, BL		;
	cp	AL,_0		;
	cpc	AH,_0		;
	brne	PC-6		;
	cpi	ZL, 3		;
	breq	PC+5		;
	inc	ZL		;
	st	Z, ZH		;
	ldi	ZH, ' '		;
	rjmp	PC-5		;
	ld	AL, Z		;
	 rcall	xmit		;
	dec	ZL		;
	brne	PC-3		;/
	ldi	AL, '.'		;.
	 rcall	xmit		;/
	mov	AL, CL		;x0.1
	rjmp	xmit		;/


div10:	clr	BL		;AH:AL = AH:AL / 10
	ldi	BH,16		;BL = AH:AL % 10
	lslw	A		;
	rol	BL		;
	cpi	BL,10		;
	brcs	PC+3		;
	subi	BL,10		;
	inc	AL		;
	dec	BH		;
	brne	PC-8		;/
	ret


get_temp:
	ldi	AL, 6
	 rcall	get_adc
	ldi	BL, high(32)		;Error check
	cpi	AL, low(32)		;
	cpc	AH, BL			;
	brcs	se_err			;
	ldi	BL, high(960)		;
	cpi	AL, low(960)		;
	cpc	AH, BL			;
	brcc	se_err			;/
	 rcall	linearize		;Linearization
	ret
se_err:	sbr	_Flags, bit1
	ret

get_adc:
	out	ADMUX, AL
	outi	ADCSR, 0b11010011
	sbis	ADCSR, ADIF
	rjmp	PC-1
	inw	A, ADC
	ret


linearize:
	ldiw	Z, th_tbl*2	;Z = th_tbl;
	cpi	AL, 32		;while(AH:AL >= 32) {
	cpc	AH, _0		;
	brcs	PC+6		;
	subi	AL, low(32)	; AH:AL -= 32;
	sbci	AH, high(32)	;
	addiw	Z, 2		; Z++;
	rjmp	PC-7		;}
	lpm			;r3:r2 = *Z; //base_t
	mov	r2, r0		;
	addiw	Z, 1		;
	lpm			;
	mov	r3, r0		;
	addiw	Z, 2		;r1:r0 = *(Z+1) - *Z; //span
	lpm			;
	mov	r1, r0		;
	subiw	Z, 1		;
	lpm			;
	sub	r0, r2		;
	sbc	r1, r3		;/
	ldi	CL, 9		;AH:AL *= span;
	sub	AH, AH		;
	ror	AH		;
	ror	AL		;
	brcc	PC+2		;
	add	AH, r0		;
	dec	CL		;
	brne	PC-5		;/
	ldi	CL, 5		;AH:AL /= 32;
	lsrw	A		;
	dec	CL		;
	brne	PC-3		;
	adc	AL, _0		;
	adc	AH, _0		;/
	add	AL, r2		;AH:AL += base_t;
	adc	AH, r3		;/
	ret

; Linearization table for 103AT/502AT (10bit A/D, 32 segments)
; When use any other thermister, rebuild this table to fit to the thermister.

th_tbl:	.dw	-700,	-490,	-358,	-273,	-207,	-152,	-105,	-62
	.dw	-22,	15,	51,	85,	118,	151,	184,	217
	.dw	250,	284,	318,	354,	392,	431,	473,	519
	.dw	569,	624,	688,	763,	856,	976,	1134,	1200
	.dw	1300



;--------------------------------------;
; zXg 1s
;
; Call: none
; Ret:  LineBuf

get_line:
	ldiw	X,LineBuf
	ldi	r19,0
rl_lp:	 rcall	receive
	st	X,r16
	cpi	r16,0x0d	; CR
	brne	PC+2
	rjmp	xmit
	cpi	r16,0x08	; BS
	brne	PC+7
	cpi	r19,0
	breq	rl_lp
	 rcall	xmit
	sbiw	XL,1
	dec	r19
	rjmp	rl_lp
	cpi	r16,' '		; SP
	brcs	rl_lp
	cpi	r19,15
	breq	rl_lp
	 rcall	xmit
	adiw	XL,1
	inc	r19
	rjmp	rl_lp

caps:	cpi	r19,'a'		; CAPS
	brcs	PC+2		; 
	subi	r19,0x20	; /
	ret


;--------------------------------------;
; ROM̕𑗐M
; Send ROM string
;
; Call: Z = top of the string (ASCIZ)
; Ret:  Z = next string

dp_str:	lpm
	adiw	ZL,1
	tst	r0
	brne	PC+2
	ret
	mov	r16,r0
	 rcall	xmit
	rjmp	PC-7

;--------------------------------------;
; 10io

dp_dec:		sbrs	AH, 7
		rjmp	PC+9
		push	AL
		ldi	AL, '-'
		 rcall	xmit
		pop	AL
		comw	A
		subiw	A, -1

		clr	DL		;digit counter
		inc	DL		;---- decimal string generating loop
		clr	CL		;AH /= 10;
		ldi	CH,16		;
		lslw	A		;
		rol	CL		;
		cpi	CL,10		;
		brcs	PC+3		;
		subi	CL,10		;
		inc	AL		;
		dec	CH		;
		brne	PC-8		;/
		subi	CL,-'0'		;Push the remander (a decimal digit)
		push	CL		;/
		cp	AL,CH		;if(var =! 0)
		cpc	AH,CH		; continue digit loop;
		brne	PC-16		;/
		pop	AL		;Put decimal string
		 rcall	xmit		;<-- Put a char (AL) to memory, console or any display device
		dec	DL		;
		brne	PC-3		;/
		ret


;--------------------------------------;
; 10i̒l擾
; Get decimal value
;
; Call: X = string pointer
; Ret:  X = updated
;         if    C=1: error
;         elsif Z=1: end of line, value=0
;         else:      r17:r16 = 16bit value

get_valu:
		clrw	A		;var = 0;
		ld	BL, X		;Skip " "
		cpi	BL, ' '		;
		brcc	PC+4		;
		clc			;
		sez			;
		ret			;
		brne	PC+3		;
		adiw	XL, 1		;
		rjmp	PC-8		;/
					;--- digit processing loop
		cpi	BL, ' '+1	;convert a numeric char into its value
		brcc	PC+4		;
		clz			;
		clc			;
		ret			;
		subi	BL, '0'		;
		brcc	PC+4		;
		sec	;error		;
		sez	;		;
		ret	;/		;
		cpi	BL, 10		;
		brcc	PC-4		;/
		ldi	BH, 17		;var = var * 10 + BL;
		sub	CL, CL		;
		brcc	PC+2		;
		subi	CL, -10		;
		lsr	CL		;
		rorw	A		;
		dec	BH		;
		brne	PC-6		;
		addw	A, B		;/
		adiw	XL, 1
		ld	BL, X
		rjmp	PC-25



;--------------------------------------;
; ISPP[ugp̃\tgEFAUART
; Software implemented UART using ISP cable

xmit:	; Transmit r16. r16 is broken
	push	r17
	push	r18
	com	r16		; Start bit
	ldi	r18,10		; bit count
	ldi	r17,BPS-1	; Wait for full bit time
	dec	r17		; 
	brne	PC-1		; /
	brcs	PC+2		; MISO = bit for send
	cbi	PORTB,MISO	; 
	brcc	PC+2		; 
	sbi	PORTB,MISO	; /
	lsr	r16		; next bit
	dec	r18		; All sent ?
	brne	PC-9		;  no, coutinue
	pop	r18
	pop	r17
	ret


receive:; Receive into r16. r17 is broken
	ldi	r16,0x80	; Shift register for receive
	sbic	PINB,MOSI	; Wait for rise edge of MOSI
	rjmp	PC-1		; 
	sbis	PINB,MOSI	; 
	rjmp	PC-1		; /
	ldi	r17,BPS/2	; Wait for half bit time
	dec	r17		; 
	brne	PC-1		; /
	ldi	r17,BPS		; Wait for full bit time
	dec	r17		; 
	brne	PC-1		; /
	lsr	r16		; Next bit
	sbis	PINB,MOSI	; Read MOSI into r16.7
	ori	r16,0x80	; /
	brcc	PC-6		; All bits received?  no, continue
	ret			;                     yes, end



;----------------------------------------------------------;

isr_tc0ovf:
	push	AL
	in	AL, SREG
	sei
	pushw	A

	mov	AL, _Tmr
	inc	AL
	andi	AL, 3
	mov	_Tmr, AL
	brne	PC+2
	sbr	_Flags, bit0

	clr	AH
	sbrc	_Flags, 2
	ldi	AH, 2
	sbrc	_Flags, 1
	ldi	AH, 1
	tst	AH
	breq	mc_on
	and	AH, AL
	breq	mc_on
	sbi	PORTA, 0
	rjmp	PC+2
mc_on:	cbi	PORTA, 0

	popw	A
	pop	AL
	out	SREG, AL
	reti



;----------------------------------------------------------;

isr_int0:
	sbrc	_Flags, 2
	rjmp	PC+7
	sbrc	_Flags, 1
	rjmp	PC+5
	sbic	PINB, 6
	rjmp	PC+3
	sbi	PORTA, 1
	reti
	cbi	PORTA, 1
	reti


;----------------------------------------------------------;
; ROMf[^

.equ	_PC	= PC

m_crlf:		.db	0x0d, 0x0a, 0

m_error:	.db	0x0a, "???", 0

m_start:	.db	0x0d,0x0d, "AVR DEBUG MONITOR", 0x0d, 0x0a, "O<pa><d> - Output port", 0x0d, 0x0a, "I<pa> - Input port", 0x0d, 0x0a, "IR<pa> - Input port repeat", 0x0d, 0x0a, "E<sa>[<d>] - Edit memory", 0x0d, 0x0a, 0

	